/*
 * Copyright (c) 2021 lumidify <nobody@lumidify.org>
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <Imlib2.h>

int
main(int argc, char *argv[]) {
	char *dst_filename;
	char *format;
	char alpha;
	int w, h, x, y;
	int orig_w, orig_h;
	Imlib_Image src, dst;
	Imlib_Load_Error error;
	if (argc < 3 || argc > 4) {
		fprintf(stderr,
		    "USAGE: croptool_crop <width>x<height>+<x>+<y> "
		    "<source filename> [destination filename]\n"
		);
		exit(1);
	}
	if (argc == 4)
		dst_filename = argv[3];
	else
		dst_filename = argv[2];
	/* FIXME: proper error checking for this (don't use sscanf) */
	if (sscanf(argv[1], "%dx%d+%d+%d", &w, &h, &x, &y) < 4) {
		fprintf(stderr, "Invalid cropping rectangle specified.\n");
		exit(1);
	}
	src = imlib_load_image(argv[2]);
	if (!src) {
		fprintf(stderr, "Unable to load image.\n");
		exit(1);
	}
	imlib_context_set_image(src);
	alpha = imlib_image_has_alpha();
	format = strdup(imlib_image_format());
	if (!format) {
		fprintf(stderr, "Unable to allocate memory.\n");
		exit(1);
	}
	orig_w = imlib_image_get_width();
	orig_h = imlib_image_get_height();
	if (w <= 0 || h <= 0 || x < 0 || y < 0 || x + w > orig_w || y + h > orig_h) {
		fprintf(stderr, "Invalid cropping rectangle specified.\n");
		exit(1);
	}

	dst = imlib_create_image(w, h);
	if (!dst) {
		fprintf(stderr, "Unable to create cropped image.\n");
		exit(1);
	}
	imlib_context_set_image(dst);
	imlib_image_set_has_alpha(alpha);
	imlib_blend_image_onto_image(src, 1, x, y, w, h, 0, 0, w, h);

	imlib_image_set_format(format);
	imlib_save_image_with_error_return(dst_filename, &error);
	if (error) {
		fprintf(stderr, "Unable to save cropped image.\n");
		exit(1);
	}

	imlib_free_image();
	imlib_context_set_image(src);
	imlib_free_image();
	free(format);

	return 0;
}
